//V1 - original release
#include <ePaper.h>
#include <Wire.h>
#include <avr/sleep.h>
#include "DS3231.h"

DS3231 RTC; //Create the DS3231 object

int EIO1pin = 9;     // Input/output pin for chip selection
int XCKpin = 10;     // Clock input pin for taking display data
int LATCHpin = 11;   // Latch pulse input pin for display data
int SLEEPBpin = 12;  // Sleep Pin for the display
int DI0pin = 13;     // Input pin for display data
int VCCpin = 8;

int nextButton = 5; //pos "T" on 3 way nav
int upButton = 6;  //pos "1" on 3 way nav
int downButton = 4;  //pos "2" on 3 way nav

//setup display with pin definitions
ePaper epaper = ePaper(EIO1pin, XCKpin, LATCHpin, SLEEPBpin, DI0pin);

int seconds; 
int minutes; 
int hours;
int dayOfWeek;
int dayOfMonth;
int month;
int year;

char *space;
char *suffix;
char *dayOfWeekChar;
char *seperator; 

char *bottomLineData;

int buttonCount = 0; //start at zero starts clock in run mode

unsigned long previousMillis = 0; // last time update
long interval = 1000; // interval at which to do something (milliseconds)

void setup() {

  //Initialize INT0 for accepting interrupts
  PORTD |= 0x04; 
  DDRD &=~ 0x04;

  Wire.begin();
  Serial.begin(9600);

  RTC.begin();
  attachInterrupt(0, displayTime, FALLING); 

  //Enable Interrupt 
  RTC.enableInterrupts(EveryMinute); //interrupt at  EverySecond, EveryMinute, EveryHour
  // or this
  //RTC.enableInterrupts(18,4,0);    // interrupt at (h,m,s)

  pinMode (nextButton, INPUT); 
  pinMode (upButton, INPUT);
  pinMode (downButton, INPUT);

  pinMode (VCCpin, OUTPUT);
  digitalWrite(VCCpin, HIGH);

  //////////////////////////////// force time setting:
  /*
    seconds = 00;
   minutes = 03;
   hours = 17; //in 24 format
   dayOfWeek = 7;
   dayOfMonth = 25;
   month = 05;
   year = 25;
   initChrono();
   */
  ///////////////////////////////

  epaper.writeTop("  EPAPER  ");
  epaper.writeBottom(" Clock v1 ");
  epaper.writeDisplay();
  delay(500);
  epaper.writeTop("  EPAPER  ");
  epaper.writeBottom(" Clock v1 ");
  epaper.writeDisplay();
  delay(500);
  epaper.writeTop("  EPAPER  ");
  epaper.writeBottom(" Clock v1 ");
  epaper.writeDisplay();
  delay(500);
  epaper.writeTop("  EPAPER  ");
  epaper.writeBottom(" Clock v1 ");
  epaper.writeDisplay();
  delay(500);
  epaper.writeTop("  EPAPER  ");
  epaper.writeBottom(" Clock v1 ");
  epaper.writeDisplay();
  delay(5000);

  epaper.deleteDisplay(); 
}

void loop() {

  //poll the DS3231 for the date and time
  get_time();
  get_date();  


  //display set menus if nextButton is pressed
  if (digitalRead(nextButton) == LOW) {

    buttonCount = buttonCount + 1;

    if (buttonCount == 7) {
      buttonCount = 0;
    }

    //debounce
    delay(600);
  }

  //////////////////////////////////////////////////////////////////////////////////////////////////
  //not in set mode, button count = 8
  //display time
  if(buttonCount == 0) {
    //Dsplay time/date - default operation

    //don't rewrite the time and date on every loop, wait x seconds (interval) and then update the time.
    //if(currentMillis - previousMillis > interval) {
    // previousMillis = currentMillis;  

    Serial.print("Button Count = 0");
    // time
    //display hours
    if (hours > 11) {
      suffix = " PM ";
    } 
    else {
      suffix = " AM ";
    }

    if (hours > 12) {
      hours -= 12;
    } 
    else if (hours == 0) {
      hours = 12;
    }

    //figure out leading spaces
    if (hours > 9) {
      space = " "; //1 space
    } 
    else {
      space = "  "; //2 spaces
    }

    if (minutes < 10) {
      seperator = "-0";
    } 
    else {
      seperator = "-";
    }

    //construct the string to be displayed
    char timeChr[10];
    sprintf(timeChr, "%s%d%s%d%s", space, hours, seperator, minutes, suffix);

    /////////////////////////////////////////////////////
    //date
    switch(dayOfWeek){
    case 1: 
      dayOfWeekChar ="Sun";
      break;
    case 2: 
      dayOfWeekChar ="Mon";
      break;
    case 3: 
      dayOfWeekChar ="Tue";
      break;
    case 4: 
      dayOfWeekChar ="Wed";
      break;
    case 5: 
      dayOfWeekChar ="Thu";
      break;
    case 6: 
      dayOfWeekChar ="Fri";
      break;
    case 7: 
      dayOfWeekChar ="Sat";
      break;
    }

    //construct the string to be displayed
    char dateChr[10];

    if ((month < 10) && (dayOfMonth < 10)) {
      sprintf(dateChr, " %s  %d/%d   ", dayOfWeekChar, month, dayOfMonth); //add an extra space to center text when month and DOM are < 10; extra spaces are intentional so no weird char pop in at the end
    }
    else {
      sprintf(dateChr, " %s %d/%d   ", dayOfWeekChar, month, dayOfMonth); //extra spaces are intentional so no weird char pop in at the end
    }

    epaper.writeTop(timeChr);
    epaper.writeBottom(dateChr);
    epaper.writeDisplay();
    // }
  }

  ////////////////////////////////////////////////////////////////////////////////////
  //enter set mode
  if(buttonCount == 1) {
    Serial.print("Button Count = 1");
    //epaper.deleteDisplay(); 
    epaper.writeTop("HOURS     ");
    if (hours < 10) {
      space = "         "; //9 long
    } 
    else {
      space = "        "; //8 long
    }
    char displayChr[10];
    sprintf(displayChr, "%d%s", hours, space);
    epaper.writeBottom(displayChr);
    epaper.writeDisplay();
  }
  if(buttonCount == 2) {
    Serial.print("Button Count = 2");
    //epaper.deleteDisplay(); 
    epaper.writeTop("MINUTES   ");
    if (minutes < 10) {
      space = "         "; //9 long
    } 
    else {
      space = "        "; //8 long
    }
    char displayChr[10];
    sprintf(displayChr, "%d%s", minutes, space);
    epaper.writeBottom(displayChr);
    epaper.writeDisplay();
  }
  if(buttonCount == 3) {
    Serial.print("Button Count = 3");
    // epaper.deleteDisplay(); 
    epaper.writeTop("WEEKDAY   ");
    switch(dayOfWeek){
    case 1: 
      dayOfWeekChar ="Sun       ";
      break;
    case 2: 
      dayOfWeekChar ="Mon       ";
      break;
    case 3: 
      dayOfWeekChar ="Tue       ";
      break;
    case 4: 
      dayOfWeekChar ="Wed       ";
      break;
    case 5: 
      dayOfWeekChar ="Thu       ";
      break;
    case 6: 
      dayOfWeekChar ="Fri       ";
      break;
    case 7: 
      dayOfWeekChar ="Sat       ";
      break;
    }   
    epaper.writeBottom(dayOfWeekChar);
    epaper.writeDisplay();
  }
  if(buttonCount == 4) {
    Serial.print("Button Count = 4");
    // epaper.deleteDisplay();
    epaper.writeTop("DAY       ");
    if (dayOfMonth < 10) {
      space = "         "; //9 long
    } 
    else {
      space = "        "; //8 long
    }
    char displayChr[10];
    sprintf(displayChr, "%d%s", dayOfMonth, space);
    epaper.writeBottom(displayChr);
    epaper.writeDisplay();
  }
  if(buttonCount == 5) {
    Serial.print("Button Count = 5");
    epaper.deleteDisplay();
    epaper.writeTop("Month     ");
    if (month < 10) {
      space = "         "; //9 long
    } 
    else {
      space = "        "; //8 long
    }
    char displayChr[10];
    sprintf(displayChr, "%d%s", month, space);
    epaper.writeBottom(displayChr);
    epaper.writeDisplay();
  }
  if(buttonCount == 6) {
    Serial.print("Button Count = 6");
    //  epaper.deleteDisplay();
    epaper.writeTop("YEAR      ");
    space = "        "; //8 long
    char displayChr[10];
    sprintf(displayChr, "%d%s", year, space);
    epaper.writeBottom(displayChr);
    epaper.writeDisplay();
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //pressing 'up' button should increment field being set
  //hours
  if ((digitalRead(upButton) == LOW) && (buttonCount == 1)) {

    delay(200);

    hours++; 
    if (hours > 23) hours = 0;

    //send update to RTC 
    set_time();
  }

  //mins
  else if ((digitalRead(upButton) == LOW) && (buttonCount == 2)) {   

    delay(200);

    minutes++;              
    if (minutes > 59) minutes = 0;

    //reset seconds to '0'
    seconds = 0;

    //send update to RTC
    set_time();
  }

  //day of week
  else if ((digitalRead(upButton) == LOW) && (buttonCount == 3)) {   

    delay(200);

    dayOfWeek++;
    if(dayOfWeek > 7) dayOfWeek = 1;

    //send update to RTC
    set_date();
  }

  //day of month
  else if ((digitalRead(upButton) == LOW) && (buttonCount == 4)) {

    delay(200);

    dayOfMonth++;              

    //if feb
    if (month == 2) {
      if (dayOfMonth > 28) dayOfMonth = 1;
    }

    //if leap year
    //still to do

    //if month has 30 days: Apr, Jun, Sep, Nov
    if ((month == 4) || (month == 6) || (month == 9) || (month == 11))  {
      if (dayOfMonth > 30) dayOfMonth = 1; 
    }

    //if month has 31 days: Jan, Mar, May, Jul, Aug, Oct, Dec
    if ((month == 1) || (month == 3) || (month == 5) || (month == 7) || (month == 8) || (month == 10)) {
      if (dayOfMonth > 31) dayOfMonth = 1;
    }

    //send update to RTC
    set_date();
  }

  //month
  else if ((digitalRead(upButton) == LOW) && (buttonCount == 5)) {

    delay(200);

    month++;              
    if (month > 12) month = 1;

    //send update to RTC
    set_date();
  }

  //year
  else if ((digitalRead(upButton) == LOW) && (buttonCount == 6)) {

    delay(200);

    year++;              

    //send update to RTC
    set_date();
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //pressing 'down' button should decrement field being set
  //hours
  if ((digitalRead(downButton) == LOW) && (buttonCount == 1)) {

    delay(200);

    if (hours == 0) hours = 23;    
    else if ((hours > 0) && (hours <= 23)) hours--;

    //send update to RTC 
    set_time();
  }

  //mins
  else if ((digitalRead(downButton) == LOW) && (buttonCount == 2)) {   

    delay(200);

    if (minutes == 0) minutes = 59;    
    else if ((minutes > 0) && (minutes <= 59)) minutes--; 

    //reset seconds to '0'
    seconds = 0;

    //send update to RTC
    set_time();
  }

  //day of week
  else if ((digitalRead(downButton) == LOW) && (buttonCount == 3)) {   

    delay(200);

    dayOfWeek--;
    if(dayOfWeek < 1 ) dayOfWeek = 7;                                            

    //send update to RTC
    set_date();
  }

  //day of month
  else if ((digitalRead(downButton) == LOW) && (buttonCount == 4)) {

    delay(200);

    dayOfMonth--;              

    //if feb
    if (month == 2) {
      if (dayOfMonth < 1) dayOfMonth = 28;
    }

    //if leap year
    //still to do

    //if month has 30 days: Apr, Jun, Sep, Nov
    if ((month == 4) || (month == 6) || (month == 9) || (month == 11))  {
      if (dayOfMonth < 1) dayOfMonth = 30; 
    }

    //if month has 31 days: Jan, Mar, May, Jul, Aug, Oct, Dec
    if ((month == 1) || (month == 3) || (month == 5) || (month == 7) || (month == 8) || (month == 10)) {
      if (dayOfMonth < 1) dayOfMonth = 31;
    }

    //send update to RTC
    set_date();
  }

  //month
  else if ((digitalRead(downButton) == LOW) && (buttonCount == 5)) {

    delay(200);

    month--;              
    if (month < 1) month = 12;

    //send update to RTC
    set_date();
  }

  //year
  else if ((digitalRead(downButton) == LOW) && (buttonCount == 6)) {

    delay(200);

    year--;              
    if (year < 13) year = 13;

    //send update to RTC
    set_date();
  }


} //end of loop

///////////////////////////////////////////////////////////////////////////////////////////
//DS3231 RTC interface
void initChrono()
{
  set_time();
  set_date();
}

void set_date()
{
  Wire.beginTransmission(104);
  Wire.write(3);
  Wire.write(decToBcd(dayOfWeek));
  Wire.write(decToBcd(dayOfMonth));
  Wire.write(decToBcd(month));
  Wire.write(decToBcd(year));
  Wire.endTransmission();
}

void get_date()
{
  Wire.beginTransmission(104); 
  Wire.write(3);//set register to 3 (day)
  Wire.endTransmission();
  Wire.requestFrom(104, 4); //get 5 bytes(day,date,month,year,control);
  dayOfWeek   = bcdToDec(Wire.read());
  dayOfMonth  = bcdToDec(Wire.read());
  month = bcdToDec(Wire.read());
  year  = bcdToDec(Wire.read());
}

void set_time()
{
  Wire.beginTransmission(104);
  Wire.write(0);
  Wire.write(decToBcd(seconds));
  Wire.write(decToBcd(minutes));
  Wire.write(decToBcd(hours));
  Wire.endTransmission();
}

void get_time()
{
  Wire.beginTransmission(104); 
  Wire.write(0);//set register to 0
  Wire.endTransmission();
  Wire.requestFrom(104, 3);//get 3 bytes (seconds,minutes,hours);
  seconds = bcdToDec(Wire.read() & 0x7f);
  minutes = bcdToDec(Wire.read());
  hours = bcdToDec(Wire.read() & 0x3f);
}

///////////////////////////////////////////////////////////////////////
byte decToBcd(byte val)
{
  return ( (val/10*16) + (val%10) );
}

byte bcdToDec(byte val)
{
  return ( (val/16*10) + (val%16) );
}









void displayTime() {

  Serial.print("Button Count = 0");
  // time
  //display hours
  if (hours > 11) {
    suffix = " PM ";
  } 
  else {
    suffix = " AM ";
  }

  if (hours > 12) {
    hours -= 12;
  } 
  else if (hours == 0) {
    hours = 12;
  }

  //figure out leading spaces
  if (hours > 9) {
    space = " "; //1 space
  } 
  else {
    space = "  "; //2 spaces
  }

  if (minutes < 10) {
    seperator = "-0";
  } 
  else {
    seperator = "-";
  }

  //construct the string to be displayed
  char timeChr[10];
  sprintf(timeChr, "%s%d%s%d%s", space, hours, seperator, minutes, suffix);

  /////////////////////////////////////////////////////
  //date
  switch(dayOfWeek){
  case 1: 
    dayOfWeekChar ="Sun";
    break;
  case 2: 
    dayOfWeekChar ="Mon";
    break;
  case 3: 
    dayOfWeekChar ="Tue";
    break;
  case 4: 
    dayOfWeekChar ="Wed";
    break;
  case 5: 
    dayOfWeekChar ="Thu";
    break;
  case 6: 
    dayOfWeekChar ="Fri";
    break;
  case 7: 
    dayOfWeekChar ="Sat";
    break;
  }

  //construct the string to be displayed
  char dateChr[10];

  if ((month < 10) && (dayOfMonth < 10)) {
    sprintf(dateChr, " %s  %d/%d   ", dayOfWeekChar, month, dayOfMonth); //add an extra space to center text when month and DOM are < 10; extra spaces are intentional so no weird char pop in at the end
  }
  else {
    sprintf(dateChr, " %s %d/%d   ", dayOfWeekChar, month, dayOfMonth); //extra spaces are intentional so no weird char pop in at the end
  }

  epaper.writeTop(timeChr);
  epaper.writeBottom(dateChr);
  epaper.writeDisplay();

  delay(50);
  epaper.writeTop(timeChr);
  epaper.writeBottom(dateChr);
  epaper.writeDisplay();

  delay(50);
  epaper.writeTop(timeChr);
  epaper.writeBottom(dateChr);
  epaper.writeDisplay();
}

















